"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.objectExtensions = exports.urlEncode = exports.compact = exports.merge = void 0;
const ExpressionError_1 = require("../ExpressionError");
function merge(value, extraArgs) {
    const [other] = extraArgs;
    if (typeof other !== 'object' || !other) {
        throw new ExpressionError_1.ExpressionExtensionError('argument of merge must be an object');
    }
    const newObject = { ...value };
    for (const [key, val] of Object.entries(other)) {
        if (!(key in newObject)) {
            newObject[key] = val;
        }
    }
    return newObject;
}
exports.merge = merge;
function isEmpty(value) {
    return Object.keys(value).length === 0;
}
function hasField(value, extraArgs) {
    const [name] = extraArgs;
    return name in value;
}
function removeField(value, extraArgs) {
    const [name] = extraArgs;
    if (name in value) {
        const newObject = { ...value };
        delete newObject[name];
        return newObject;
    }
    return value;
}
function removeFieldsContaining(value, extraArgs) {
    const [match] = extraArgs;
    if (typeof match !== 'string') {
        throw new ExpressionError_1.ExpressionExtensionError('argument of removeFieldsContaining must be an string');
    }
    const newObject = { ...value };
    for (const [key, val] of Object.entries(value)) {
        if (typeof val === 'string' && val.includes(match)) {
            delete newObject[key];
        }
    }
    return newObject;
}
function keepFieldsContaining(value, extraArgs) {
    const [match] = extraArgs;
    if (typeof match !== 'string') {
        throw new ExpressionError_1.ExpressionExtensionError('argument of keepFieldsContaining must be an string');
    }
    const newObject = { ...value };
    for (const [key, val] of Object.entries(value)) {
        if (typeof val === 'string' && !val.includes(match)) {
            delete newObject[key];
        }
    }
    return newObject;
}
function compact(value) {
    const newObj = {};
    for (const [key, val] of Object.entries(value)) {
        if (val !== null && val !== undefined) {
            if (typeof val === 'object') {
                newObj[key] = compact(val);
            }
            else {
                newObj[key] = val;
            }
        }
    }
    return newObj;
}
exports.compact = compact;
function urlEncode(value) {
    return new URLSearchParams(value).toString();
}
exports.urlEncode = urlEncode;
exports.objectExtensions = {
    typeName: 'Object',
    functions: {
        isEmpty,
        merge,
        hasField,
        removeField,
        removeFieldsContaining,
        keepFieldsContaining,
        compact,
        urlEncode,
    },
};
//# sourceMappingURL=ObjectExtensions.js.map