"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringExtensions = void 0;
const ExpressionError = __importStar(require("../ExpressionError"));
const crypto_js_1 = __importDefault(require("crypto-js"));
const js_base64_1 = require("js-base64");
const transliteration_1 = require("transliteration");
const hashFunctions = {
    md5: crypto_js_1.default.MD5,
    sha1: crypto_js_1.default.SHA1,
    sha224: crypto_js_1.default.SHA224,
    sha256: crypto_js_1.default.SHA256,
    sha384: crypto_js_1.default.SHA384,
    sha512: crypto_js_1.default.SHA512,
    sha3: crypto_js_1.default.SHA3,
    ripemd160: crypto_js_1.default.RIPEMD160,
};
const CURRENCY_REGEXP = /(\u004c\u0065\u006b|\u060b|\u0024|\u0192|\u20bc|\u0042\u0072|\u0042\u005a\u0024|\u0024\u0062|\u004b\u004d|\u0050|\u043b\u0432|\u0052\u0024|\u17db|\u00a5|\u20a1|\u006b\u006e|\u20b1|\u004b\u010d|\u006b\u0072|\u0052\u0044\u0024|\u00a3|\u20ac|\u00a2|\u0051|\u004c|\u0046\u0074|\u20b9|\u0052\u0070|\ufdfc|\u20aa|\u004a\u0024|\u20a9|\u20ad|\u0434\u0435\u043d|\u0052\u004d|\u20a8|\u20ae|\u004d\u0054|\u0043\u0024|\u20a6|\u0042\u002f\u002e|\u0047\u0073|\u0053\u002f\u002e|\u007a\u0142|\u006c\u0065\u0069|\u20bd|\u0414\u0438\u043d\u002e|\u0053|\u0052|\u0043\u0048\u0046|\u004e\u0054\u0024|\u0e3f|\u0054\u0054\u0024|\u20ba|\u20b4|\u0024\u0055|\u0042\u0073|\u20ab|\u005a\u0024)/gu;
const DOMAIN_REGEXP = /^[a-zA-Z0-9][a-zA-Z0-9-]{1,61}[a-zA-Z0-9](?:\.[a-zA-Z]{2,})+$/;
const EMAIL_REGEXP = /(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@(?<domain>(\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))/;
const URL_REGEXP = /https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{2,}\b([-a-zA-Z0-9()\[\]@:%_\+.~#?&//=]*)/;
const CHAR_TEST_REGEXP = /\p{L}/u;
const PUNC_TEST_REGEXP = /[!?.]/;
const TRUE_VALUES = ['true', '1', 't', 'yes', 'y'];
const FALSE_VALUES = ['false', '0', 'f', 'no', 'n'];
function encrypt(value, extraArgs) {
    const [format = 'MD5'] = extraArgs;
    if (format.toLowerCase() === 'base64') {
        return (0, js_base64_1.encode)(value);
    }
    const hashFunction = hashFunctions[format.toLowerCase()];
    if (!hashFunction) {
        throw new ExpressionError.ExpressionExtensionError(`Unknown encrypt type ${format}. Available types are: ${Object.keys(hashFunctions)
            .map((s) => s.toUpperCase())
            .join(', ')}, and Base64.`);
    }
    return hashFunction(value.toString()).toString();
}
function getOnlyFirstCharacters(value, extraArgs) {
    const [end] = extraArgs;
    if (typeof end !== 'number') {
        throw new ExpressionError.ExpressionExtensionError('getOnlyFirstCharacters() requires a argument');
    }
    return value.slice(0, end);
}
function isBlank(value) {
    return value === '';
}
function isPresent(value) {
    return !isBlank(value);
}
function length(value) {
    return value.length;
}
function removeMarkdown(value) {
    let output = value;
    try {
        output = output.replace(/^([\s\t]*)([*\-+]|\d\.)\s+/gm, '$1');
        output = output
            .replace(/\n={2,}/g, '\n')
            .replace(/~~/g, '')
            .replace(/`{3}.*\n/g, '');
        output = output
            .replace(/<[\w|\s|=|'|"|:|(|)|,|;|/|0-9|.|-]+[>|\\>]/g, '')
            .replace(/^[=-]{2,}\s*$/g, '')
            .replace(/\[\^.+?\](: .*?$)?/g, '')
            .replace(/\s{0,2}\[.*?\]: .*?$/g, '')
            .replace(/!\[.*?\][[(].*?[\])]/g, '')
            .replace(/\[(.*?)\][[(].*?[\])]/g, '$1')
            .replace(/>/g, '')
            .replace(/^\s{1,2}\[(.*?)\]: (\S+)( ".*?")?\s*$/g, '')
            .replace(/^#{1,6}\s*([^#]*)\s*(#{1,6})?/gm, '$1')
            .replace(/([*_]{1,3})(\S.*?\S)\1/g, '$2')
            .replace(/(`{3,})(.*?)\1/gm, '$2')
            .replace(/^-{3,}\s*$/g, '')
            .replace(/`(.+?)`/g, '$1')
            .replace(/\n{2,}/g, '\n\n');
    }
    catch (e) {
        return value;
    }
    return output;
}
function sayHi(value) {
    return `hi ${value}`;
}
function stripTags(value) {
    return value.replace(/<[^>]*>?/gm, '');
}
function toDate(value) {
    return new Date(value.toString());
}
function urlDecode(value, extraArgs) {
    const [entireString = false] = extraArgs;
    if (entireString) {
        return decodeURI(value.toString());
    }
    return decodeURIComponent(value.toString());
}
function urlEncode(value, extraArgs) {
    const [entireString = false] = extraArgs;
    if (entireString) {
        return encodeURI(value.toString());
    }
    return encodeURIComponent(value.toString());
}
function toInt(value, extraArgs) {
    const [radix] = extraArgs;
    return parseInt(value.replace(CURRENCY_REGEXP, ''), radix);
}
function toFloat(value) {
    return parseFloat(value.replace(CURRENCY_REGEXP, ''));
}
function quote(value, extraArgs) {
    const [quoteChar = '"'] = extraArgs;
    return `${quoteChar}${value
        .replace(/\\/g, '\\\\')
        .replace(new RegExp(`\\${quoteChar}`, 'g'), `\\${quoteChar}`)}${quoteChar}`;
}
function isTrue(value) {
    return TRUE_VALUES.includes(value.toLowerCase());
}
function isFalse(value) {
    return FALSE_VALUES.includes(value.toLowerCase());
}
function isNumeric(value) {
    return !isNaN(value) && !isNaN(parseFloat(value));
}
function isUrl(value) {
    let url;
    try {
        url = new URL(value);
    }
    catch (_error) {
        return false;
    }
    return url.protocol === 'http:' || url.protocol === 'https:';
}
function isDomain(value) {
    return DOMAIN_REGEXP.test(value);
}
function isEmail(value) {
    return EMAIL_REGEXP.test(value);
}
function stripSpecialChars(value) {
    return (0, transliteration_1.transliterate)(value, { unknown: '?' });
}
function toTitleCase(value) {
    return value.replace(/\w\S*/g, (v) => v.charAt(0).toLocaleUpperCase() + v.slice(1));
}
function toSentenceCase(value) {
    let current = value.slice();
    let buffer = '';
    while (CHAR_TEST_REGEXP.test(current)) {
        const charIndex = current.search(CHAR_TEST_REGEXP);
        current =
            current.slice(0, charIndex) +
                current[charIndex].toLocaleUpperCase() +
                current.slice(charIndex + 1).toLocaleLowerCase();
        const puncIndex = current.search(PUNC_TEST_REGEXP);
        if (puncIndex === -1) {
            buffer += current;
            current = '';
            break;
        }
        buffer += current.slice(0, puncIndex + 1);
        current = current.slice(puncIndex + 1);
    }
    return buffer;
}
function toSnakeCase(value) {
    return value
        .toLocaleLowerCase()
        .replace(/[ \-]/g, '_')
        .replace(/[\u2000-\u206F\u2E00-\u2E7F\\'!"#$%&()*+,.\/:;<=>?@\[\]^`{|}~]/g, '');
}
function extractEmail(value) {
    const matched = EMAIL_REGEXP.exec(value);
    if (!matched) {
        return undefined;
    }
    return matched[0];
}
function extractDomain(value) {
    var _a;
    if (isEmail(value)) {
        const matched = EMAIL_REGEXP.exec(value);
        if (!matched) {
            return undefined;
        }
        return (_a = matched.groups) === null || _a === void 0 ? void 0 : _a.domain;
    }
    else if (isUrl(value)) {
        return new URL(value).hostname;
    }
    return undefined;
}
function extractUrl(value) {
    const matched = URL_REGEXP.exec(value);
    if (!matched) {
        return undefined;
    }
    return matched[0];
}
exports.stringExtensions = {
    typeName: 'String',
    functions: {
        encrypt,
        hash: encrypt,
        getOnlyFirstCharacters,
        removeMarkdown,
        sayHi,
        stripTags,
        toBoolean: isTrue,
        toDate,
        toDecimalNumber: toFloat,
        toFloat,
        toInt,
        toWholeNumber: toInt,
        toSentenceCase,
        toSnakeCase,
        toTitleCase,
        urlDecode,
        urlEncode,
        quote,
        stripSpecialChars,
        length,
        isDomain,
        isEmail,
        isTrue,
        isFalse,
        isNotTrue: isFalse,
        isNumeric,
        isUrl,
        isURL: isUrl,
        isBlank,
        isPresent,
        extractEmail,
        extractDomain,
        extractUrl,
    },
};
//# sourceMappingURL=StringExtensions.js.map