"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkflowDataProxy = exports.isResourceLocatorValue = void 0;
const luxon_1 = require("luxon");
const jmespath = __importStar(require("jmespath"));
const NodeHelpers = __importStar(require("./NodeHelpers"));
const ExpressionError_1 = require("./ExpressionError");
const utils_1 = require("./utils");
function isResourceLocatorValue(value) {
    return Boolean(typeof value === 'object' && value && 'mode' in value && 'value' in value && '__rl' in value);
}
exports.isResourceLocatorValue = isResourceLocatorValue;
const SCRIPTING_NODE_TYPES = [
    'n8n-nodes-base.function',
    'n8n-nodes-base.functionItem',
    'n8n-nodes-base.code',
];
const isScriptingNode = (nodeName, workflow) => {
    const node = workflow.getNode(nodeName);
    return node && SCRIPTING_NODE_TYPES.includes(node.type);
};
class WorkflowDataProxy {
    constructor(workflow, runExecutionData, runIndex, itemIndex, activeNodeName, connectionInputData, siblingParameters, mode, defaultTimezone, additionalKeys, executeData, defaultReturnRunIndex = -1, selfData = {}) {
        this.activeNodeName = activeNodeName;
        this.workflow = workflow;
        this.runExecutionData = isScriptingNode(activeNodeName, workflow)
            ? (0, utils_1.deepCopy)(runExecutionData)
            : runExecutionData;
        this.connectionInputData = isScriptingNode(activeNodeName, workflow)
            ? (0, utils_1.deepCopy)(connectionInputData)
            : connectionInputData;
        this.defaultReturnRunIndex = defaultReturnRunIndex;
        this.runIndex = runIndex;
        this.itemIndex = itemIndex;
        this.siblingParameters = siblingParameters;
        this.mode = mode;
        this.defaultTimezone = defaultTimezone;
        this.timezone = this.workflow.settings.timezone || this.defaultTimezone;
        this.selfData = selfData;
        this.additionalKeys = additionalKeys;
        this.executeData = executeData;
        luxon_1.Settings.defaultZone = this.timezone;
    }
    nodeContextGetter(nodeName) {
        var _a, _b, _c;
        const that = this;
        const node = this.workflow.nodes[nodeName];
        if (!((_a = that.runExecutionData) === null || _a === void 0 ? void 0 : _a.executionData) && that.connectionInputData.length > 1) {
            return {};
        }
        if (!((_b = that.runExecutionData) === null || _b === void 0 ? void 0 : _b.executionData) && !((_c = that.runExecutionData) === null || _c === void 0 ? void 0 : _c.resultData)) {
            throw new ExpressionError_1.ExpressionError("The workflow hasn't been executed yet, so you can't reference any context data", {
                runIndex: that.runIndex,
                itemIndex: that.itemIndex,
            });
        }
        return new Proxy({}, {
            ownKeys(target) {
                if (Reflect.ownKeys(target).length === 0) {
                    Object.assign(target, NodeHelpers.getContext(that.runExecutionData, 'node', node));
                }
                return Reflect.ownKeys(target);
            },
            getOwnPropertyDescriptor(k) {
                return {
                    enumerable: true,
                    configurable: true,
                };
            },
            get(target, name, receiver) {
                name = name.toString();
                const contextData = NodeHelpers.getContext(that.runExecutionData, 'node', node);
                return contextData[name];
            },
        });
    }
    selfGetter() {
        const that = this;
        return new Proxy({}, {
            ownKeys(target) {
                return Reflect.ownKeys(target);
            },
            get(target, name, receiver) {
                name = name.toString();
                return that.selfData[name];
            },
        });
    }
    nodeParameterGetter(nodeName) {
        const that = this;
        const node = this.workflow.nodes[nodeName];
        return new Proxy(node.parameters, {
            ownKeys(target) {
                return Reflect.ownKeys(target);
            },
            getOwnPropertyDescriptor(k) {
                return {
                    enumerable: true,
                    configurable: true,
                };
            },
            get(target, name, receiver) {
                name = name.toString();
                let returnValue;
                if (name[0] === '&') {
                    const key = name.slice(1);
                    if (!that.siblingParameters.hasOwnProperty(key)) {
                        throw new Error(`Could not find sibling parameter "${key}" on node "${nodeName}"`);
                    }
                    returnValue = that.siblingParameters[key];
                }
                else {
                    if (!node.parameters.hasOwnProperty(name)) {
                        return undefined;
                    }
                    returnValue = node.parameters[name];
                }
                if (isResourceLocatorValue(returnValue)) {
                    if (returnValue.__regex && typeof returnValue.value === 'string') {
                        const expr = new RegExp(returnValue.__regex);
                        const extracted = expr.exec(returnValue.value);
                        if (extracted && extracted.length >= 2) {
                            returnValue = extracted[1];
                        }
                        else {
                            return returnValue.value;
                        }
                    }
                    else {
                        returnValue = returnValue.value;
                    }
                }
                if (typeof returnValue === 'string' && returnValue.charAt(0) === '=') {
                    return that.workflow.expression.getParameterValue(returnValue, that.runExecutionData, that.runIndex, that.itemIndex, that.activeNodeName, that.connectionInputData, that.mode, that.timezone, that.additionalKeys, that.executeData);
                }
                return returnValue;
            },
        });
    }
    getNodeExecutionData(nodeName, shortSyntax = false, outputIndex, runIndex) {
        const that = this;
        let executionData;
        if (!shortSyntax) {
            if (that.runExecutionData === null) {
                throw new ExpressionError_1.ExpressionError("The workflow hasn't been executed yet, so you can't reference any output data", {
                    runIndex: that.runIndex,
                    itemIndex: that.itemIndex,
                });
            }
            if (!that.runExecutionData.resultData.runData.hasOwnProperty(nodeName)) {
                if (that.workflow.getNode(nodeName)) {
                    throw new ExpressionError_1.ExpressionError(`no data, execute "${nodeName}" node first`, {
                        runIndex: that.runIndex,
                        itemIndex: that.itemIndex,
                    });
                }
                throw new ExpressionError_1.ExpressionError(`"${nodeName}" node doesn't exist`, {
                    runIndex: that.runIndex,
                    itemIndex: that.itemIndex,
                });
            }
            runIndex = runIndex === undefined ? that.defaultReturnRunIndex : runIndex;
            runIndex =
                runIndex === -1 ? that.runExecutionData.resultData.runData[nodeName].length - 1 : runIndex;
            if (that.runExecutionData.resultData.runData[nodeName].length <= runIndex) {
                throw new ExpressionError_1.ExpressionError(`Run ${runIndex} of node "${nodeName}" not found`, {
                    runIndex: that.runIndex,
                    itemIndex: that.itemIndex,
                });
            }
            const taskData = that.runExecutionData.resultData.runData[nodeName][runIndex].data;
            if (taskData.main === null || !taskData.main.length || taskData.main[0] === null) {
                throw new ExpressionError_1.ExpressionError('No data found from "main" input.', {
                    runIndex: that.runIndex,
                    itemIndex: that.itemIndex,
                });
            }
            if (outputIndex === undefined) {
                const nodeConnection = that.workflow.getNodeConnectionIndexes(that.activeNodeName, nodeName, 'main');
                if (nodeConnection === undefined) {
                    throw new ExpressionError_1.ExpressionError(`connect "${that.activeNodeName}" to "${nodeName}"`, {
                        runIndex: that.runIndex,
                        itemIndex: that.itemIndex,
                    });
                }
                outputIndex = nodeConnection.sourceIndex;
            }
            if (outputIndex === undefined) {
                outputIndex = 0;
            }
            if (taskData.main.length <= outputIndex) {
                throw new ExpressionError_1.ExpressionError(`Node "${nodeName}" has no branch with index ${outputIndex}.`, {
                    runIndex: that.runIndex,
                    itemIndex: that.itemIndex,
                });
            }
            executionData = taskData.main[outputIndex];
        }
        else {
            executionData = that.connectionInputData;
        }
        return executionData;
    }
    nodeDataGetter(nodeName, shortSyntax = false) {
        const that = this;
        const node = this.workflow.nodes[nodeName];
        return new Proxy({ binary: undefined, data: undefined, json: undefined }, {
            get(target, name, receiver) {
                name = name.toString();
                if (!node) {
                    throw new ExpressionError_1.ExpressionError(`"${nodeName}" node doesn't exist`);
                }
                if (['binary', 'data', 'json'].includes(name)) {
                    const executionData = that.getNodeExecutionData(nodeName, shortSyntax, undefined);
                    if (executionData.length <= that.itemIndex) {
                        throw new ExpressionError_1.ExpressionError(`No data found for item-index: "${that.itemIndex}"`, {
                            runIndex: that.runIndex,
                            itemIndex: that.itemIndex,
                        });
                    }
                    if (['data', 'json'].includes(name)) {
                        return executionData[that.itemIndex].json;
                    }
                    if (name === 'binary') {
                        const returnData = {};
                        if (!executionData[that.itemIndex].binary) {
                            return returnData;
                        }
                        const binaryKeyData = executionData[that.itemIndex].binary;
                        for (const keyName of Object.keys(binaryKeyData)) {
                            returnData[keyName] = {};
                            const binaryData = binaryKeyData[keyName];
                            for (const propertyName in binaryData) {
                                if (propertyName === 'data') {
                                    continue;
                                }
                                returnData[keyName][propertyName] = binaryData[propertyName];
                            }
                        }
                        return returnData;
                    }
                }
                else if (name === 'context') {
                    return that.nodeContextGetter(nodeName);
                }
                else if (name === 'parameter') {
                    return that.nodeParameterGetter(nodeName);
                }
                else if (name === 'runIndex') {
                    if (that.runExecutionData === null ||
                        !that.runExecutionData.resultData.runData[nodeName]) {
                        return -1;
                    }
                    return that.runExecutionData.resultData.runData[nodeName].length - 1;
                }
                return Reflect.get(target, name, receiver);
            },
        });
    }
    envGetter() {
        const that = this;
        return new Proxy({}, {
            get(target, name, receiver) {
                if (typeof process === 'undefined' ||
                    process.env.N8N_BLOCK_ENV_ACCESS_IN_NODE === 'true') {
                    throw new ExpressionError_1.ExpressionError('access to env vars denied', {
                        causeDetailed: 'If you need access please contact the administrator to remove the environment variable ‘N8N_BLOCK_ENV_ACCESS_IN_NODE‘',
                        runIndex: that.runIndex,
                        itemIndex: that.itemIndex,
                        failExecution: true,
                    });
                }
                return process.env[name.toString()];
            },
        });
    }
    prevNodeGetter() {
        const allowedValues = ['name', 'outputIndex', 'runIndex'];
        const that = this;
        return new Proxy({}, {
            ownKeys(target) {
                return allowedValues;
            },
            getOwnPropertyDescriptor(k) {
                return {
                    enumerable: true,
                    configurable: true,
                };
            },
            get(target, name, receiver) {
                var _a;
                if (!((_a = that.executeData) === null || _a === void 0 ? void 0 : _a.source)) {
                    return undefined;
                }
                const sourceData = that.executeData.source.main[0];
                if (name === 'name') {
                    return sourceData.previousNode;
                }
                if (name === 'outputIndex') {
                    return sourceData.previousNodeOutput || 0;
                }
                if (name === 'runIndex') {
                    return sourceData.previousNodeRun || 0;
                }
                return Reflect.get(target, name, receiver);
            },
        });
    }
    workflowGetter() {
        const allowedValues = ['active', 'id', 'name'];
        const that = this;
        return new Proxy({}, {
            ownKeys(target) {
                return allowedValues;
            },
            getOwnPropertyDescriptor(k) {
                return {
                    enumerable: true,
                    configurable: true,
                };
            },
            get(target, name, receiver) {
                if (allowedValues.includes(name.toString())) {
                    const value = that.workflow[name];
                    if (value === undefined && name === 'id') {
                        throw new ExpressionError_1.ExpressionError('save workflow to view', {
                            description: 'Please save the workflow first to use $workflow',
                            runIndex: that.runIndex,
                            itemIndex: that.itemIndex,
                            failExecution: true,
                        });
                    }
                    return value;
                }
                return Reflect.get(target, name, receiver);
            },
        });
    }
    nodeGetter() {
        const that = this;
        return new Proxy({}, {
            get(target, name, receiver) {
                const nodeName = name.toString();
                if (that.workflow.getNode(nodeName) === null) {
                    throw new ExpressionError_1.ExpressionError(`"${nodeName}" node doesn't exist`, {
                        runIndex: that.runIndex,
                        itemIndex: that.itemIndex,
                    });
                }
                return that.nodeDataGetter(nodeName);
            },
        });
    }
    getDataProxy() {
        const that = this;
        const getNodeOutput = (nodeName, branchIndex, runIndex) => {
            let executionData;
            if (nodeName === undefined) {
                executionData = that.connectionInputData;
            }
            else {
                branchIndex = branchIndex || 0;
                runIndex = runIndex === undefined ? -1 : runIndex;
                executionData = that.getNodeExecutionData(nodeName, false, branchIndex, runIndex);
            }
            return executionData;
        };
        const jmespathWrapper = (data, query) => {
            if (!Array.isArray(data) && typeof data === 'object') {
                return jmespath.search({ ...data }, query);
            }
            return jmespath.search(data, query);
        };
        const createExpressionError = (message, context) => {
            if (isScriptingNode(that.activeNodeName, that.workflow) && (context === null || context === void 0 ? void 0 : context.functionOverrides)) {
                message = context.functionOverrides.message || message;
                context.description = context.functionOverrides.description || context.description;
                context.messageTemplate = undefined;
            }
            if (context === null || context === void 0 ? void 0 : context.nodeCause) {
                const nodeName = context.nodeCause;
                const pinData = this.workflow.getPinDataOfNode(nodeName);
                if (pinData) {
                    if (!context) {
                        context = {};
                    }
                    message = `‘Node ${nodeName}‘ must be unpinned to execute`;
                    context.messageTemplate = undefined;
                    context.description = `To fetch the data for the expression, you must unpin the node <strong>'${nodeName}'</strong> and execute the workflow again.`;
                    context.descriptionTemplate = `To fetch the data for the expression under '%%PARAMETER%%', you must unpin the node <strong>'${nodeName}'</strong> and execute the workflow again.`;
                }
                if (context.moreInfoLink && (pinData || isScriptingNode(nodeName, that.workflow))) {
                    const moreInfoLink = ' <a target="_blank" href="https://docs.n8n.io/data/data-mapping/data-item-linking/item-linking-errors/">More info</a>';
                    context.description += moreInfoLink;
                    context.descriptionTemplate += moreInfoLink;
                }
            }
            return new ExpressionError_1.ExpressionError(message, {
                runIndex: that.runIndex,
                itemIndex: that.itemIndex,
                failExecution: true,
                ...context,
            });
        };
        const getPairedItem = (destinationNodeName, incomingSourceData, pairedItem) => {
            let taskData;
            let sourceData = incomingSourceData;
            if (pairedItem.sourceOverwrite) {
                sourceData = pairedItem.sourceOverwrite;
            }
            if (typeof pairedItem === 'number') {
                pairedItem = {
                    item: pairedItem,
                };
            }
            const previousNodeHasPinData = sourceData && this.workflow.getPinDataOfNode(sourceData.previousNode) !== undefined;
            let currentPairedItem = pairedItem;
            let nodeBeforeLast;
            while (!previousNodeHasPinData &&
                sourceData !== null &&
                destinationNodeName !== sourceData.previousNode) {
                taskData =
                    that.runExecutionData.resultData.runData[sourceData.previousNode][(sourceData === null || sourceData === void 0 ? void 0 : sourceData.previousNodeRun) || 0];
                const previousNodeOutput = sourceData.previousNodeOutput || 0;
                if (previousNodeOutput >= taskData.data.main.length) {
                    throw createExpressionError('Can’t get data for expression', {
                        messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                        functionOverrides: {
                            message: 'Can’t get data',
                        },
                        nodeCause: nodeBeforeLast,
                        description: 'Apologies, this is an internal error. See details for more information',
                        causeDetailed: 'Referencing a non-existent output on a node, problem with source data',
                        type: 'internal',
                    });
                }
                if (pairedItem.item >= taskData.data.main[previousNodeOutput].length) {
                    throw createExpressionError('Can’t get data for expression', {
                        messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                        functionality: 'pairedItem',
                        functionOverrides: {
                            message: 'Can’t get data',
                        },
                        nodeCause: nodeBeforeLast,
                        description: `In node ‘<strong>${nodeBeforeLast}</strong>’, output item ${currentPairedItem.item || 0} ${sourceData.previousNodeRun
                            ? `of run ${(sourceData.previousNodeRun || 0).toString()} `
                            : ''}points to an input item on node ‘<strong>${sourceData.previousNode}</strong>‘ that doesn’t exist.`,
                        type: 'invalid pairing info',
                        moreInfoLink: true,
                    });
                }
                const itemPreviousNode = taskData.data.main[previousNodeOutput][pairedItem.item];
                if (itemPreviousNode.pairedItem === undefined) {
                    throw createExpressionError('Can’t get data for expression', {
                        messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                        functionality: 'pairedItem',
                        functionOverrides: {
                            message: 'Can’t get data',
                        },
                        nodeCause: sourceData.previousNode,
                        description: `To fetch the data from other nodes that this expression needs, more information is needed from the node ‘<strong>${sourceData.previousNode}</strong>’`,
                        causeDetailed: `Missing pairedItem data (node ‘${sourceData.previousNode}’ probably didn’t supply it)`,
                        type: 'no pairing info',
                        moreInfoLink: true,
                    });
                }
                if (Array.isArray(itemPreviousNode.pairedItem)) {
                    const results = itemPreviousNode.pairedItem
                        .map((item) => {
                        try {
                            const itemInput = item.input || 0;
                            if (itemInput >= taskData.source.length) {
                                throw new Error('Not found');
                            }
                            return getPairedItem(destinationNodeName, taskData.source[itemInput], item);
                        }
                        catch (error) {
                            return null;
                        }
                    })
                        .filter((result) => result !== null);
                    if (results.length !== 1) {
                        throw createExpressionError('Invalid expression', {
                            messageTemplate: 'Invalid expression under ‘%%PARAMETER%%’',
                            functionality: 'pairedItem',
                            functionOverrides: {
                                description: `The code uses data in the node ‘<strong>${destinationNodeName}</strong>’ but there is more than one matching item in that node`,
                                message: 'Invalid code',
                            },
                            description: `The expression uses data in the node ‘<strong>${destinationNodeName}</strong>’ but there is more than one matching item in that node`,
                            type: 'multiple matches',
                        });
                    }
                    return results[0];
                }
                currentPairedItem = pairedItem;
                if (typeof itemPreviousNode.pairedItem === 'number') {
                    pairedItem = {
                        item: itemPreviousNode.pairedItem,
                    };
                }
                else {
                    pairedItem = itemPreviousNode.pairedItem;
                }
                const itemInput = pairedItem.input || 0;
                if (itemInput >= taskData.source.length) {
                    if (taskData.source.length === 0) {
                        throw createExpressionError('Invalid expression', {
                            messageTemplate: 'Invalid expression under ‘%%PARAMETER%%’',
                            functionality: 'pairedItem',
                            functionOverrides: {
                                description: `The code uses data in the node ‘<strong>${destinationNodeName}</strong>’ but there is no path back to it. Please check this node is connected to it (there can be other nodes in between).`,
                                message: 'Invalid code',
                            },
                            description: `The expression uses data in the node ‘<strong>${destinationNodeName}</strong>’ but there is no path back to it. Please check this node is connected to it (there can be other nodes in between).`,
                            type: 'no connection',
                            moreInfoLink: true,
                        });
                    }
                    throw createExpressionError('Can’t get data for expression', {
                        messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                        functionality: 'pairedItem',
                        functionOverrides: {
                            message: 'Can’t get data',
                        },
                        nodeCause: nodeBeforeLast,
                        description: `In node ‘<strong>${sourceData.previousNode}</strong>’, output item ${currentPairedItem.item || 0} of ${sourceData.previousNodeRun
                            ? `of run ${(sourceData.previousNodeRun || 0).toString()} `
                            : ''}points to a branch that doesn’t exist.`,
                        type: 'invalid pairing info',
                    });
                }
                nodeBeforeLast = sourceData.previousNode;
                sourceData = taskData.source[pairedItem.input || 0] || null;
                if (pairedItem.sourceOverwrite) {
                    sourceData = pairedItem.sourceOverwrite;
                }
            }
            if (sourceData === null) {
                throw createExpressionError('Can’t get data for expression', {
                    messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                    functionality: 'pairedItem',
                    functionOverrides: {
                        message: 'Can’t get data',
                    },
                    nodeCause: nodeBeforeLast,
                    description: 'Could not resolve, proably no pairedItem exists',
                    type: 'no pairing info',
                    moreInfoLink: true,
                });
            }
            taskData =
                that.runExecutionData.resultData.runData[sourceData.previousNode][(sourceData === null || sourceData === void 0 ? void 0 : sourceData.previousNodeRun) || 0];
            const previousNodeOutput = sourceData.previousNodeOutput || 0;
            if (previousNodeOutput >= taskData.data.main.length) {
                throw createExpressionError('Can’t get data for expression', {
                    messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                    functionality: 'pairedItem',
                    functionOverrides: {
                        message: 'Can’t get data',
                    },
                    description: 'Item points to a node output which does not exist',
                    causeDetailed: `The sourceData points to a node output ‘${previousNodeOutput}‘ which does not exist on node ‘${sourceData.previousNode}‘ (output node did probably supply a wrong one)`,
                    type: 'invalid pairing info',
                });
            }
            if (pairedItem.item >= taskData.data.main[previousNodeOutput].length) {
                throw createExpressionError('Can’t get data for expression', {
                    messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                    functionality: 'pairedItem',
                    functionOverrides: {
                        message: 'Can’t get data',
                    },
                    nodeCause: nodeBeforeLast,
                    description: `In node ‘<strong>${nodeBeforeLast}</strong>’, output item ${currentPairedItem.item || 0} ${sourceData.previousNodeRun
                        ? `of run ${(sourceData.previousNodeRun || 0).toString()} `
                        : ''}points to an input item on node ‘<strong>${sourceData.previousNode}</strong>‘ that doesn’t exist.`,
                    type: 'invalid pairing info',
                    moreInfoLink: true,
                });
            }
            return taskData.data.main[previousNodeOutput][pairedItem.item];
        };
        const base = {
            $: (nodeName) => {
                if (!nodeName) {
                    throw createExpressionError('When calling $(), please specify a node');
                }
                const referencedNode = that.workflow.getNode(nodeName);
                if (referencedNode === null) {
                    throw createExpressionError(`"${nodeName}" node doesn't exist`);
                }
                return new Proxy({}, {
                    ownKeys(target) {
                        return [
                            'pairedItem',
                            'itemMatching',
                            'item',
                            'first',
                            'last',
                            'all',
                            'context',
                            'params',
                        ];
                    },
                    get(target, property, receiver) {
                        var _a;
                        if (['pairedItem', 'itemMatching', 'item'].includes(property)) {
                            const pairedItemMethod = (itemIndex) => {
                                var _a;
                                if (itemIndex === undefined) {
                                    if (property === 'itemMatching') {
                                        throw createExpressionError('Missing item index for .itemMatching()', {
                                            itemIndex,
                                        });
                                    }
                                    itemIndex = that.itemIndex;
                                }
                                const executionData = that.connectionInputData;
                                const pairedItem = executionData[itemIndex].pairedItem;
                                if (pairedItem === undefined) {
                                    throw createExpressionError('Can’t get data for expression', {
                                        messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                                        functionality: 'pairedItem',
                                        functionOverrides: {
                                            description: `To fetch the data from other nodes that this code needs, more information is needed from the node ‘<strong>${that.activeNodeName}</strong>‘`,
                                            message: 'Can’t get data',
                                        },
                                        description: `To fetch the data from other nodes that this expression needs, more information is needed from the node ‘<strong>${that.activeNodeName}</strong>‘`,
                                        causeDetailed: `Missing pairedItem data (node ‘${that.activeNodeName}‘ probably didn’t supply it)`,
                                        itemIndex,
                                    });
                                }
                                if (!((_a = that.executeData) === null || _a === void 0 ? void 0 : _a.source)) {
                                    throw createExpressionError('Can’t get data for expression', {
                                        messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                                        functionality: 'pairedItem',
                                        functionOverrides: {
                                            message: 'Can’t get data',
                                        },
                                        description: 'Apologies, this is an internal error. See details for more information',
                                        causeDetailed: 'Missing sourceData (probably an internal error)',
                                        itemIndex,
                                    });
                                }
                                const parentNodes = that.workflow.getParentNodes(that.activeNodeName);
                                if (!parentNodes.includes(nodeName)) {
                                    throw createExpressionError('Invalid expression', {
                                        messageTemplate: 'Invalid expression under ‘%%PARAMETER%%’',
                                        functionality: 'pairedItem',
                                        functionOverrides: {
                                            description: `The code uses data in the node <strong>‘${nodeName}’</strong> but there is no path back to it. Please check this node is connected to it (there can be other nodes in between).`,
                                            message: `No path back to node ‘${nodeName}’`,
                                        },
                                        description: `The expression uses data in the node <strong>‘${nodeName}’</strong> but there is no path back to it. Please check this node is connected to it (there can be other nodes in between).`,
                                        itemIndex,
                                    });
                                }
                                const sourceData = that.executeData.source.main[pairedItem.input || 0];
                                return getPairedItem(nodeName, sourceData, pairedItem);
                            };
                            if (property === 'item') {
                                return pairedItemMethod();
                            }
                            return pairedItemMethod;
                        }
                        if (property === 'first') {
                            return (branchIndex, runIndex) => {
                                const executionData = getNodeOutput(nodeName, branchIndex, runIndex);
                                if (executionData[0])
                                    return executionData[0];
                                return undefined;
                            };
                        }
                        if (property === 'last') {
                            return (branchIndex, runIndex) => {
                                const executionData = getNodeOutput(nodeName, branchIndex, runIndex);
                                if (!executionData.length)
                                    return undefined;
                                if (executionData[executionData.length - 1]) {
                                    return executionData[executionData.length - 1];
                                }
                                return undefined;
                            };
                        }
                        if (property === 'all') {
                            return (branchIndex, runIndex) => getNodeOutput(nodeName, branchIndex, runIndex);
                        }
                        if (property === 'context') {
                            return that.nodeContextGetter(nodeName);
                        }
                        if (property === 'params') {
                            return (_a = that.workflow.getNode(nodeName)) === null || _a === void 0 ? void 0 : _a.parameters;
                        }
                        return Reflect.get(target, property, receiver);
                    },
                });
            },
            $input: new Proxy({}, {
                ownKeys(target) {
                    return ['all', 'context', 'first', 'item', 'last', 'params'];
                },
                getOwnPropertyDescriptor(k) {
                    return {
                        enumerable: true,
                        configurable: true,
                    };
                },
                get(target, property, receiver) {
                    var _a, _b;
                    if (property === 'item') {
                        return that.connectionInputData[that.itemIndex];
                    }
                    if (property === 'first') {
                        return (...args) => {
                            if (args.length) {
                                throw createExpressionError('$input.first() should have no arguments');
                            }
                            const result = that.connectionInputData;
                            if (result[0]) {
                                return result[0];
                            }
                            return undefined;
                        };
                    }
                    if (property === 'last') {
                        return (...args) => {
                            if (args.length) {
                                throw createExpressionError('$input.last() should have no arguments');
                            }
                            const result = that.connectionInputData;
                            if (result.length && result[result.length - 1]) {
                                return result[result.length - 1];
                            }
                            return undefined;
                        };
                    }
                    if (property === 'all') {
                        return () => {
                            const result = that.connectionInputData;
                            if (result.length) {
                                return result;
                            }
                            return [];
                        };
                    }
                    if (['context', 'params'].includes(property)) {
                        if (!((_a = that.executeData) === null || _a === void 0 ? void 0 : _a.source)) {
                            throw createExpressionError('Can’t get data for expression', {
                                messageTemplate: 'Can’t get data for expression under ‘%%PARAMETER%%’ field',
                                functionOverrides: {
                                    message: 'Can’t get data',
                                },
                                description: 'Apologies, this is an internal error. See details for more information',
                                causeDetailed: 'Missing sourceData (probably an internal error)',
                                runIndex: that.runIndex,
                            });
                        }
                        const sourceData = that.executeData.source.main[0];
                        if (property === 'context') {
                            return that.nodeContextGetter(sourceData.previousNode);
                        }
                        if (property === 'params') {
                            return (_b = that.workflow.getNode(sourceData.previousNode)) === null || _b === void 0 ? void 0 : _b.parameters;
                        }
                    }
                    return Reflect.get(target, property, receiver);
                },
            }),
            $binary: {},
            $data: {},
            $env: this.envGetter(),
            $evaluateExpression: (expression, itemIndex) => {
                itemIndex = itemIndex || that.itemIndex;
                return that.workflow.expression.getParameterValue(`=${expression}`, that.runExecutionData, that.runIndex, itemIndex, that.activeNodeName, that.connectionInputData, that.mode, that.timezone, that.additionalKeys, that.executeData);
            },
            $item: (itemIndex, runIndex) => {
                const defaultReturnRunIndex = runIndex === undefined ? -1 : runIndex;
                const dataProxy = new WorkflowDataProxy(this.workflow, this.runExecutionData, this.runIndex, itemIndex, this.activeNodeName, this.connectionInputData, that.siblingParameters, that.mode, that.defaultTimezone, that.additionalKeys, that.executeData, defaultReturnRunIndex);
                return dataProxy.getDataProxy();
            },
            $items: (nodeName, outputIndex, runIndex) => {
                if (nodeName === undefined) {
                    nodeName = that.prevNodeGetter().name;
                    const node = this.workflow.nodes[nodeName];
                    let result = that.connectionInputData;
                    if (node.executeOnce === true) {
                        result = result.slice(0, 1);
                    }
                    if (result.length) {
                        return result;
                    }
                    return [];
                }
                outputIndex = outputIndex || 0;
                runIndex = runIndex === undefined ? -1 : runIndex;
                return that.getNodeExecutionData(nodeName, false, outputIndex, runIndex);
            },
            $json: {},
            $node: this.nodeGetter(),
            $self: this.selfGetter(),
            $parameter: this.nodeParameterGetter(this.activeNodeName),
            $prevNode: this.prevNodeGetter(),
            $runIndex: this.runIndex,
            $mode: this.mode,
            $workflow: this.workflowGetter(),
            $itemIndex: this.itemIndex,
            $now: luxon_1.DateTime.now(),
            $today: luxon_1.DateTime.now().set({ hour: 0, minute: 0, second: 0, millisecond: 0 }),
            $jmesPath: jmespathWrapper,
            DateTime: luxon_1.DateTime,
            Interval: luxon_1.Interval,
            Duration: luxon_1.Duration,
            ...that.additionalKeys,
            $jmespath: jmespathWrapper,
            $position: this.itemIndex,
            $thisItem: that.connectionInputData[that.itemIndex],
            $thisItemIndex: this.itemIndex,
            $thisRunIndex: this.runIndex,
        };
        return new Proxy(base, {
            get(target, name, receiver) {
                var _a, _b;
                if (['$data', '$json'].includes(name)) {
                    return (_a = that.nodeDataGetter(that.activeNodeName, true)) === null || _a === void 0 ? void 0 : _a.json;
                }
                if (name === '$binary') {
                    return (_b = that.nodeDataGetter(that.activeNodeName, true)) === null || _b === void 0 ? void 0 : _b.binary;
                }
                return Reflect.get(target, name, receiver);
            },
        });
    }
}
exports.WorkflowDataProxy = WorkflowDataProxy;
//# sourceMappingURL=WorkflowDataProxy.js.map